var fs = require('fs');
var path = require('path');

var utils = module.exports = {};

utils.lo = require('lodash');

utils.crc32 = require('./crc32');

utils.cleanBuffer = function(length) {
  var buffer = new Buffer(length);

  buffer.fill(0);

  return buffer;
};

utils.collectStream = function(stream, callback) {
  var collection = [];
  var size = 0;

  stream.on('error', callback);

  stream.on('data', function(chunk) {
    collection.push(chunk);
    size += chunk.length;
  });

  stream.on('end', function() {
    var buffer = new Buffer(size, 'utf8');
    var offset = 0;

    collection.forEach(function(data) {
      data.copy(buffer, offset);
      offset += data.length;
    });

    callback(null, buffer);
  });
};

utils.convertDateTimeDos = function(input) {
  var d = {};
  d.year = (input >> 25) + 1980;
  d.month = ((input >> 21) & 0xF) - 1;
  d.day = (input >> 16) & 0xF;

  d.hour = (input >> 11) & 0xF;
  d.minutes = (input >> 5) & 0xF;
  d.seconds = (input & 0xF) * 2;

  return new Date(d.year, d.month, d.day, d.hour, d.minutes, d.seconds);
};

utils.convertDateTimeOctal = function(input) {
  input = parseInt(input, 10) * 1000;

  return new Date(input);
};

utils.dosDateTime = function(d) {
  d = (d instanceof Date) ? d : new Date();

  var year = d.getFullYear();

  if (year < 1980) {
    return (1<<21) | (1<<16);
  }

  return ((year-1980) << 25) | ((d.getMonth()+1) << 21) | (d.getDate() << 16) |
    (d.getHours() << 11) | (d.getMinutes() << 5) | (d.getSeconds() >> 1);
};

utils.octalDateTime = function(d) {
  d = (d instanceof Date) ? d : new Date();

  return Math.round(d / 1000).toString(8);
};

utils.padNumber = function(num, bytes, base) {
  num = num.toString(base || 8);
  return this.repeat('0', bytes - num.length) + num;
};

utils.repeat = function(pattern, count) {
  if (count < 1) {
    return '';
  }

  var result = '';

  while (count > 0) {
    if (count & 1) {
      result += pattern;
    }

    count >>= 1, pattern += pattern;
  }

  return result;
};

utils.unixifyPath = function(filepath) {
  if (process.platform === 'win32') {
    return filepath.replace(/\\/g, '/');
  } else {
    return filepath;
  }
};